/*
 * File : dlInfoXfer.h
 * 
 * Description : This file contains the class declaration for the
 * dlInfoXfer class.
 */

#ifndef _dlInfoXfer_h_
#define _dlInfoXfer_h_

#include "dlStateMachine.h"
#include "types.h"

/*
 * Class : dlInfoXfer
 *
 * Description : This class inherits from the dlStateMachine base
 * class, and provides the behaviour for the dlInfoXfer state of
 * the data-link layer.
 * This class redefines methods to handle DL_DataReq, DL_ResetReq,
 * DL_DisconnectReq primitives, and to handle SABMs, DISCs, UAs,
 * DMs, Iframes, RRs, RNRs, REJs, FRMRs. Also provided is the
 * functionality to handle Timer T1 expiring, and erroneous frames
 *	with undefined control byte, Iframes with excess length, frames
 * 	with incorrect length, frames with unsolicited Fbit = 1. 
 */

class dlInfoXfer : public dlStateMachine {

 public:
	dlInfoXfer(dllayer *dl, Boolean reset);
	virtual ~dlInfoXfer() {}
	virtual dlStateMachine *DlDataReq(int &errorCode); 
	virtual dlStateMachine *DlResetReq(int &errorCode); 
	virtual dlStateMachine *DlDisconnectReq(int &errorCode); 

	virtual dlStateMachine *HandleSabm(int &errorCode); 
	virtual dlStateMachine *HandleDisc(int &errorCode); 
	virtual dlStateMachine *HandleUa(int &errorCode); 
	virtual dlStateMachine *HandleDm(int &errorCode); 

	virtual dlStateMachine *HandleIframe(int &errorCode);
	virtual dlStateMachine *HandleRr(int &errorCode); 
	virtual dlStateMachine *HandleRnr(int &errorCode); 
	virtual dlStateMachine *HandleRej(int &errorCode); 
	virtual dlStateMachine *HandleFrmr(int &errorCode); 

	virtual dlStateMachine *HandleT1Timer(int &errorCode);

	virtual dlStateMachine *HandleUndefined(int &errorCode); 
	virtual dlStateMachine *HandleUnsolFBit(int &errorCode);
	virtual dlStateMachine *HandleIncorrectLength(int &errorCode);
	virtual dlStateMachine *HandleIfrLengthExcess(int &errorCode);

	virtual void PrintState();

 protected:

	virtual int GetSupervisoryFrame();
	void SendStateResponse();
	void	CheckT1Status();
	dlStateMachine *HandleFrmrCondition(FrmrConditionType type,
				int &errorCode); 
	dlStateMachine *LinkReset(int &errorCode);
	void	Transmit(Boolean retransmit);
};

/*
 * MACRO : CHANGE_TO_INFOXFER
 *
 * Description : Convenience macro to change state to dlInfoXfer.
 *	The argument to the macro is passed on as the second
 * 	argument to the constructor.
 *	Creates a new state object, deletes the current
 *  object, returns the new state object.
 */

#define CHANGE_TO_INFOXFER(a) 							\
	dlStateMachine *newState = new dlInfoXfer(_dl,a); 	\
	delete this;										\
	return newState;

#endif
