/*
 * File : list.C
 *
 * Description : This file contains the definitions of the
 * member functions of the listMember and list classes.
 *
 * Portions of this work are derived from A C++ Toolkit, which is 
 * Copyright (C) 1991 by Jonathan S. Shapiro, and are used with
 * permission. A C++ Toolkit is published by Prentice Hall, Inc.
 *
 */

#include "list.h"

/*
 * Function : ~listMember
 *
 * Description : Destructor. Removes listMember from list,
 * and rearranges the pointers.
 */

listMember::~listMember()
{
	if (_next) _next->_prev = _prev;
	if (_prev) _prev->_next = _next;
	_next = 0;
	_prev = 0;

}


/*
 * Function : Append 
 *
 * Description : Appends a listMember object to the list.
 */

list&
list::Append(listMember *l)
{
	if (_tail) {
		_tail->_next = l;
		l->_prev = _tail;
	}
	else
		_head = l;

	_tail = l;

	return *this;
}

/*
 * Function : Prepend 
 *
 * Description : Prepends a listMember object to the list.
 */

list&
list::Prepend(listMember *l)
{
	if (_head) {
		_head->_prev = l;
		l->_next = _head;
	}
	else
		_tail = l;

	_head = l;

	return *this;
}

/*
 * Function : Remove
 *
 * Description : Removes a listMember object from the list.
 * Rearranges pointers.
 */

list&
list::Remove(listMember *l)
{
	if (l == _head)
		_head = _head->_next;

	if ( l == _tail)
		_tail = _tail->_prev;

	if (l->_next)
		l->_next->_prev = l->_prev;

	if (l->_prev )
		l->_prev->_next = l->_next;

	l->_next =0;
	l->_prev = 0;

	return *this;
}

/*
 * Function : CleanUpMarked
 *
 * Description : This function runs through the list
 * and removes any listMember objects which have been
 * marked for removal.
 */

list&
list::CleanUpMarked()
{
	listMember *lm;

	lm = _head;

	while ( lm != 0 ) {

		if ( lm->IsMarked() == 0 ) {
			lm = lm->_next;
			continue;
		}

		if ( lm == _head )
			_head = lm->_next;

		if ( lm == _tail )
			_tail = lm->_prev;

		if ( lm->_prev )
			lm->_prev->_next = lm->_next;

		if ( lm->_next )
			lm->_next->_prev = lm->_prev;

		listMember *tmp = lm->_next;
		delete lm;
		lm = tmp;
	}
	return *this;
}

/*
 * Function : Length
 *
 * Description: Computes and returns the length of the
 * list.
 */

int
list::Length()
{
	int len=0;
	for (listMember *lm = _head; lm != 0; lm = lm->_next)
		len++;
	return len;
}

/*
 * Function : GetNthMember
 *
 * Description : Given a number 'n', the function returns
 * the nth listMember object in the list.
 */

listMember *
list::GetNthMember(int n)
{
	listMember *lm;
	lm = _tail;

	for(int i=0; lm != 0; i++, lm = lm->_prev) {
		if ( i == n )
			break;
	}
		
	return lm;
}

