/*
 * File : phlayer.h
 *
 * Description : This file contains the class declaration for phlayer.
 */

#ifndef _phlayer_h_
#define _phlayer_h_

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include "types.h"
#include "layer.h"
#include "phEvents.h"
#include "inputHandler.h"

class dataPacket;

/*
 * Some constants..
 */

const unsigned char PH_ACTIVATE = 'a';
const unsigned char PH_DEACTIVATE = 'b';

/*
 * Class : phlayer
 *
 * Description : The class phlayer provides the functionality 
 * for implementing a physical layer. phlayer inherits from 
 * inputHandler, since the physical layer is receiving input 
 * from Unix IPC.
 * The phlayer layer inherits from the layer class to provide 
 * the standard layer interface.
 *
 * Attributes:
 * phStateMachine *_sm : A pointer to the current state object.
 * dataPacket *_lastRecdPkt : Pointer to the last received 
 *		data packet.
 * dataPacket *_pktToSend : A pointer to the data packet to 
 *		transmit.
 * int	_xmitfd : The file descriptor of the pipe to transmit 
 *		data on.
 * int	_recvfd : The file descriptor of the pipe to receive 
 *		data on.
 * char *_xmitLinkName : The name of the transmit pipe.
 * char *_recvLinkLane : The name of the receive pipe.
 */

class phlayer : public layer, public inputHandler {

 friend class phStateMachine;
 friend class phActive;
 friend class phInactive;

 public:
	phlayer(char *xmitLink, char *recvLink);
	~phlayer();

	virtual void Command(int cmd, int & errorCode);
	virtual void Command(int cmd, int & errorCode, int reason);
	virtual void Command(int cmd, int & errorCode, dataPacket *data);

	virtual void EventCallback();
	int 		 Init();

 private:
	int 			Initialize();
	phStateMachine 	*_sm;

	void	ProcessEvent(int event, int &errorCode);
	int 	SendCommand(unsigned char cmd);
	int 	OpenLink(char *,int);
	void 	CloseLink();
	int 	DataRequest(dataPacket *data);

	dataPacket *_lastRecdPkt;
	dataPacket *_pktToSend;

	int		_xmitfd, _recvfd;
	char 	*_xmitLinkName;
	char 	*_recvLinkName;
};

#endif
